<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\ORM\EntityRepository;

class Sponsor extends EntityRepository {

    public function basicSponsor($sponsor,$organizationId){

        $query = <<<SQL
                SELECT S.ID ID,
                S.BUSINESS_PARTNER AS BUSINESS_PARTNER,
                S.NAME NAME,
                (CASE
                WHEN S.STATUS = 'A' THEN 'Y'
                WHEN S.STATUS = 'I' THEN 'N'
                END) AS STATUS,
                PR.ID AS TYPE,
                PR.CONTROL AS TYPE_NAME,
                PR.LABEL AS TYPE_LABEL,
                BP.PASSWORD ,BP.PASSWORD AS PASSWORD_CONFIRM,
                BP.LOGIN AS LOGIN,
                S.NICK_NAME AS NICK_NAME,
                S.DOCUMENT_NUMBER AS DOCUMENT_NUMBER,
                (CASE
                WHEN S.DOCUMENT_TYPE = 'J' THEN 'Y'
                WHEN S.DOCUMENT_TYPE = 'F' THEN 'N'
                END) AS DOCUMENT_TYPE,
                S.DESCRIPTION AS DESCRIPTION
                FROM SPONSOR S
                JOIN BUSINESS_PARTNER BP ON( S.BUSINESS_PARTNER = BP.ID)
                INNER JOIN PARTNER_PROFILE PP ON (PP.BUSINESS_PARTNER = BP.ID)
                INNER JOIN PROFILE PR ON (PR.ID = PP.PROFILE)
                WHERE S.ID = :SPONSOR
                AND S.ORGANIZATION = :ORGANIZATION
SQL;
        $params = array(
            ":ORGANIZATION" => $organizationId,
            ":SPONSOR" => $sponsor
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $sponsorId
     * @param $organizationId
     * @return array
     */
    public function getCalendarSponsor($sponsorId, $organizationId)
    {
        $sql = "SELECT CE.ID ,C.NAME ,C.STATUS ,C.BEGIN_WEEK ,C.END_WEEK ,C.HOUR,TO_CHAR(C.BEGIN_HOUR, 'HH24:MI') AS BEGIN_HOUR,TO_CHAR(C.END_HOUR, 'HH24:MI') AS END_HOUR,C.WEEKLY_HOURS,C.MONTHLY_DAYS
                 FROM CALENDAR_ENTITY CE JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
                WHERE CE.ID_TABLE_ENTITY = '_Sponsor_$sponsorId'
                  AND C.ORGANIZATION = :organization";
        $params = array(
            ":organization" => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($sql,$params);
    }
    /**
     * @param $sponsorId
     * @param $organizationId
     * @return array
     */
    public function getContactSponsor($sponsorId, $organizationId)
    {
        $sql = "SELECT ID AS CONTACT ,NAME ,PHONE ,EMAIL ,ADDRESS ,DISTRICT ,CITY ,ZIP_CODE ,STATE ,COUNTRY ,STATUS ,ID
                 FROM CONTACT
                WHERE SPONSOR = $sponsorId
                  AND ORGANIZATION =  $organizationId";
        return $this->getEntityManager()->getConnection()->fetchAll($sql);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getExceptionSponsor($sponsor, $organization)
    {
        $query = <<<SQL
        SELECT CO.ID,CO.CALENDAR,CO.NAME,CO.STATUS,CO.COMMITMENT_TYPE AS TYPE,
                    TO_CHAR(CO.BEGIN_DATE,'DD/MM/YYYY') as BEGIN_DATE_LIST,
                    TO_CHAR(CO.END_DATE,'DD/MM/YYYY') as END_DATE_LIST,
                    TO_CHAR(CO.BEGIN_DATE,'YYYY-MM-DD') as BEGIN_DATE,
                    TO_CHAR(CO.END_DATE,'YYYY-MM-DD') as END_DATE,
                    TO_CHAR(CO.BEGIN_TIME_BREAK,'YYYY-MM-DD') as BEGIN_DATE_BREAK,
                    TO_CHAR(CO.END_TIME_BREAK,'YYYY-MM-DD') as END_DATE_BREAK,
                    CO.UNIT,CO.TIME_BREAK
                 FROM CALENDAR_ENTITY CE JOIN CALENDAR C ON (C.CALENDAR_ENTITY = CE.ID)
                 JOIN COMMITMENT CO ON(CO.CALENDAR = C.ID)
                WHERE  CE.ORGANIZATION = $organization
                AND CE.ID_TABLE_ENTITY ='_Sponsor_$sponsor'
SQL;
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getSponsorProjectData($sponsor, $organization)
    {
        $query = <<<SQL
              SELECT P.NAME ,BP.NAME AS MANAGER,P.ID,PA.PARENT
                FROM PROJECT P
                JOIN RESOURCE_PROJECT PPS ON(P.ID = PPS.PROJECT)
                JOIN RESOURCE_T RS ON(RS.ID = PPS.RESOURCE_C)
                LEFT JOIN SPONSOR SPO ON(SPO.BUSINESS_PARTNER = RS.BUSINESS_PARTNER)
                JOIN RESOURCE_PROJECT PPG ON(P.ID = PPG.PROJECT)
                JOIN RESOURCE_T RG ON(RG.ID = PPG.RESOURCE_C)
                LEFT JOIN BUSINESS_PARTNER BP ON(BP.ID = RG.BUSINESS_PARTNER)
                LEFT JOIN (
                  SELECT P.ID AS PARENT FROM PROJECT P
                  JOIN RESOURCE_PROJECT PPS ON(P.ID = PPS.PROJECT)
                  JOIN RESOURCE_T RS ON(RS.ID = PPS.RESOURCE_C)
                  LEFT JOIN SPONSOR SPO ON(SPO.BUSINESS_PARTNER = RS.BUSINESS_PARTNER)
                  WHERE PPS.TYPE = 'S'
                  AND P.ORGANIZATION = :ORGANIZATION
                  AND SPO.ID = :SPONSOR
                  AND P.STATUS <> 'D')PA ON (P.PARENT = PA.PARENT)
                WHERE PPS.TYPE = 'S'
                AND PPG.TYPE = 'M'
                AND P.ORGANIZATION = :ORGANIZATION
                AND SPO.ID = :SPONSOR
                AND P.STATUS <> 'D'
SQL;
        $param = array(
            'ORGANIZATION' => $organization,
            'SPONSOR' => $sponsor,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query,$param);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getAdvancedSponsor($sponsor, $organization)
    {
        $sql = "SELECT
                S.NICK_NAME NICK_NAME,
                S.DOCUMENT_NUMBER DOCUMENT_NUMBER,
                S.DOCUMENT_TYPE DOCUMENT_TYPE,
                S.DESCRIPTION DESCRIPTION
                FROM SPONSOR S JOIN BUSINESS_PARTNER BP ON( S.BUSINESS_PARTNER = BP.ID)
                WHERE S.ID = $sponsor
                AND S.ORGANIZATION = $organization";

        return $this->getEntityManager()->getConnection()->fetchAll($sql);
    }
    /**
     * @param $sponsor
     * @param $organization
     * @return array
     */
    public function getBasicSponsorEdit($sponsor, $organization)
    {
        $sql = "SELECT S.ID ID,
                S.BUSINESS_PARTNER BUSINESS_PARTNER,
                S.NAME NAME,
                S.STATUS STATUS,
                P.ID TYPE,
                P.NAME TYPE_NAME,
                BP.LOGIN LOGIN,
                BP.PASSWORD PASSWORD,
                BP.PASSWORD PASSWORD_CONFIRM
                FROM SPONSOR S JOIN BUSINESS_PARTNER BP ON( S.BUSINESS_PARTNER = BP.ID)
                               JOIN PARTNER_PROFILE PP  ON( PP.BUSINESS_PARTNER = BP.ID) 
                               JOIN PROFILE P           ON( P.ID = PP.PROFILE) 
                WHERE S.ID = $sponsor
                AND S.ORGANIZATION = $organization";

        return $this->getEntityManager()->getConnection()->fetchAll($sql);
    }
}